/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 * Contains some contributions under the Thrift Software License.
 * Please see doc/old-thrift-license.txt in the Thrift distribution for
 * details.
 */

namespace rs transit

include "CityServices.thrift"
include "Vehicles.thrift"

const Vehicles.Capacity DEFAULT4WHEELCAPACITY = 30

enum Powertrain {
  DIESEL = 0
  BIO_DIESEL = 1
  COMPRESSED_NATURAL_GAS = 2
  TROLLEY = 3
  HYBRID = 4
  BATTERY = 5
}

struct Bus {
  1: Vehicles.VehicleIdentifier identifier
  2: Vehicles.Capacity capacity
  3: Powertrain powertrain
  4: list<Vehicles.Material> materials
}

struct Route {
  1: string routeId
  2: list<CityServices.TransitImprovements> improvements
}

service Garage {
    Bus upgradeBus(1: Bus bus)

    list<CityServices.TransitImprovements> improvementsForRoute(1: Route route)
}