/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package thrift

import (
	"testing"
	"testing/quick"
)

type poolTest int

func TestPoolReset(t *testing.T) {
	p := newPool(nil, func(elem *poolTest) {
		*elem = 0
	})
	f := func(i int) (passed bool) {
		pt := p.get()
		defer func() {
			p.put(&pt)
			if pt != nil {
				t.Errorf("Expected pt to be nil after put, got %#v", pt)
				passed = false
			}
		}()
		if *pt != 0 {
			t.Errorf("Expected *pt to be reset to 0 after get, got %d", *pt)
		}
		*pt = poolTest(i)
		return !t.Failed()
	}
	if err := quick.Check(f, nil); err != nil {
		t.Error(err)
	}
}
